/******************************************************************************
URL.java

CVS Information:

  $Source: /homes/dsl/Master/cif/CIF-Comm-Java-Current/libraries/nexus/ParseURL.java,v $
  $Date: 1998/10/13 20:18:20 $
  $Revision: 1.1 $
  $State: Exp $
  $Author: toonen $
******************************************************************************/

package nexus;


class ParseURL
{
    static
    class Bad_Parameter_Exception
    extends
	Exception
    {
	public
	Bad_Parameter_Exception(
	    int					parameter_ordinal)
	{
	    this.parameter_ordinal = parameter_ordinal;
	}
    
	public
	Bad_Parameter_Exception(
	    int					parameter_ordinal,
	    String				description)
	{
	    super(description);
	    this.parameter_ordinal = parameter_ordinal;
	}

	public
	int
	get_Parameter_Ordinal()
	{
	    return parameter_ordinal;
	}

	private int				parameter_ordinal;
    }

    static
    class Internal_Error
    extends
	Error
    {
	public
	Internal_Error()
	{
	}

	public
	Internal_Error(
	    String				description)
	{
	    super(description);
	}
    }
    
    /**
     *
     */
    public
    ParseURL(
	String				url)
    throws
	Bad_Parameter_Exception
    {
	/*
	 * Initialize member fields
	 */
	protocol = null;
	host = null;
	port = 0;
	parameters = null;
	
	/*
	 * Extract protocol field out of URL
	 */
	int				pos;

	url = new String(url);
	url.trim();
	pos = url.indexOf(':');
	if (pos == -1)
	{
	    throw new
		Bad_Parameter_Exception(1, "bad protocol field");
	}

	try
	{
	    protocol = url.substring(0, pos);
	    url = url.substring(pos + 1);
	}
	catch (StringIndexOutOfBoundsException e)
	{
	    throw new
		Internal_Error("URL.URL() caught an unexpected " +
			       "StringIndexOutOfBoundsException");
	}

	
	/*
	 * If nothing remains, then URL precisely matches the form of
	 * "x-cif-*:"
	 */
	if (url.length() == 0)
	{
	    return;
	}
	

	/*
	 * Looking for "//" before hostname; if it's not found, then the URL
	 * doesn't match the final acceptable form of
	 * "x-cif-*://[host][:port][/...]"
	 */
	if (url.regionMatches(0, "//", 0, 2) == false)
	{
	    throw new
		Bad_Parameter_Exception(1, "incorrect format");
	}

	try
	{
	    url = url.substring(2, url.length());
	}
	catch (StringIndexOutOfBoundsException e)
	{
	    throw new
		Internal_Error("URL.URL() caught an unexpected " +
			       "StringIndexOutOfBoundsException");
	}
	
	
	/*
	 * Extract host and port fields, and protocol parameters
	 */
	int				pos_colon;
	int				pos_slash;

	pos_colon = url.indexOf(':');
	if (pos_colon == -1)
	{
	    pos_colon = url.length();
	}
	
	pos_slash = url.indexOf('/');
	if (pos_slash == -1)
	{
	    pos_slash = url.length();
	}

	if (pos_colon < pos_slash)
	{
	    pos = pos_colon;
	}
	else
	{
	    pos = pos_slash;
	}
	
	try
	{
	    if (pos > 0)
	    {
		host = url.substring(0, pos);

		try
		{
		    port = Integer.parseInt(host);
		    
		    if (pos_colon < pos_slash)
		    {
			throw new Bad_Parameter_Exception(
			    1, "incorrect hostname / port number format");
		    }
		    
		    host = null;
		}
		catch(NumberFormatException e)
		{
		}
	    }
	}
	catch (StringIndexOutOfBoundsException e)
	{
	    throw new
		Internal_Error("URL.URL() caught an unexpected " +
			       "StringIndexOutOfBoundsException");
	}

	if (url.regionMatches(pos, ":", 0, 1) == true)
	{
	    try
	    {
		port = Integer.parseInt(url.substring(pos + 1, pos_slash));
	    }
	    catch (NumberFormatException e)
	    {
		throw new
		    Bad_Parameter_Exception(1, "bad port number (NaN)");
	    }
	    catch (StringIndexOutOfBoundsException e)
	    {
		throw new
		    Internal_Error("URL.URL() caught an unexpected " +
				   "StringIndexOutOfBoundsException");
	    }

	    pos = pos_slash;
	}

	if (port < 0 || port > 65535)
	{
	    throw new
		Bad_Parameter_Exception(
		    1, "bad port number ((0 =< port =< 65535) is false");
	}
	    
	if (pos + 1 < url.length())
	{
	    try
	    {
		parameters = url.substring(pos + 1);
	    }
	    catch (StringIndexOutOfBoundsException e)
	    {
	    }
	}
    }


    /**
     *
     */
    public
    String
    get_Protocol()
    {
	return protocol;
    }


    /**
     *
     */
    public
    String
    get_Hostname()
    {
	return host;
    }

    
    /**
     *
     */
    public
    int
    get_Port()
    {
	return port;
    }

    
    /**
     *
     */
    public
    String
    get_Parameters()
    {
	return parameters;
    }


    /**
     *
     */
    public
    String
    get_String()
    {
	String					url;

	url = protocol + "://";
	if (host != null)
	{
	    url = url + host;
	}

	if (port > 0)
	{
	    url = url + ":" + port;
	}

	if (parameters != null)
	{
	    url = url + "/" + parameters;
	}

	return url;
    }

    /**************************************************************************
    **************************************************************************/

    
    /*
     * Internal data members
     */
    private String			protocol;
    private String			host;
    private int				port;
    private String			parameters;

    
    /**************************************************************************
    **************************************************************************/

    public static
    void
    main(String [] args)
    {
	ParseURL				url;
	
	try
	{
	    url = new ParseURL(args[0]);
	}
	catch (Bad_Parameter_Exception e)
	{
	    System.err.println(
		"Bad Parameter Exception (" + e.get_Parameter_Ordinal()
		+ "): " + e.getMessage() + "\n");
	    e.printStackTrace();
	    return;
	}

	System.out.println("String:    " + url.get_String());
	System.out.println("Protocol:  " + url.get_Protocol());
	System.out.println("Hostname:  " + url.get_Hostname());
	System.out.println("Port:      " + url.get_Port());
	System.out.println("Parameters: " + url.get_Parameters());
    }
    
}
