/*
 * PutBuffer
 *
 * TODO: Change this to use a linked list of byte[] when you do check_buffer_size()
 */

package nexus;

public class PutBuffer extends Object
{
    private Nexus	nexus;
    private byte[]	buffer;
    private int  	current_location;
    private int 	num_direct_puts;

    private static final int NEXUS_MAX_LIBA_SIZE = 64;
    private static final int NEXUS_MAX_TRANSFORM_INFO_SIZE = 64;
    private static final int FIXED_HEADER_SIZE = 64;
    private static final int RESERVED_HEADER_SIZE = (NEXUS_MAX_LIBA_SIZE
                                                   + NEXUS_MAX_TRANSFORM_INFO_SIZE
                                                   + FIXED_HEADER_SIZE);
    
    /*
     * PutBuffer()
     */
    public PutBuffer(Nexus nexus,
		     int buffer_size,
		     int num_direct_puts)
    {
	this.nexus = nexus;
	buffer = new byte[RESERVED_HEADER_SIZE + buffer_size];
	this.num_direct_puts = num_direct_puts;
	current_location = RESERVED_HEADER_SIZE;
    } /* PutBuffer() */


    /*
     * get_buffer()
     */
    public byte[] get_buffer()
    {
	return(buffer);
    } /* get_buffer() */

    
    /*
     * get_buffer_length()
     */
    public int get_buffer_length()
    {
	return(buffer.length);
    } /* get_buffer_length() */


    /*
     * get_current_location()
     */
    public int get_current_location()
    {
	return(current_location);
    } /* get_current_location() */

    
    /*	
     * get_num_direct_puts()
     */
    public int get_num_direct_puts()
    {
	return(num_direct_puts);
    }    


    /*
     * get_header_size()
     */
    public int get_header_size()
    {
	return(RESERVED_HEADER_SIZE);
    } /* get_header_buffer() */
    

    /*
     * check_buffer_size()
     */
    public boolean check_buffer_size(int size_needed,
				     int size_increment,
				     int num_direct_puts_needed,
				     int num_direct_puts_increment)
				     
    {
	if (size_needed > buffer.length - current_location)
	{
	    byte[] new_buf;
	    int new_size;

	    if (size_increment <= 0)
	    {
		return(false);
	    }
	    
	    new_size = buffer.length;
	    while (new_size < size_needed)
	    {
		new_size += size_increment;
	    }

	    new_buf = new byte[new_size];
	    for (int i = RESERVED_HEADER_SIZE;
		 i < RESERVED_HEADER_SIZE + current_location;
		 i++)
	    {
		new_buf[i] = buffer[i];
	    }

	    buffer = new_buf;
	}

	/* TODO: Add direct support */
	
	return(true);
    } /* check_buffer_size() */


    /*
     * check_for_overrun()
     */
    private void check_for_overrun(long put_size)
	throws BufferOverrunException
    {
	if (current_location + put_size > buffer.length)
	{
	    throw(new BufferOverrunException());
	}
    } /* check_for_overrun() */

    
    /*
     * put_TYPE(value)
     */
    public void put_float(float value)
	throws BufferOverrunException
    {
	int size = DataConversion.sizeof_float(1);
	check_for_overrun(size);
	DataConversion.put_float(value, buffer, current_location);
	current_location += size;
    }
    public void put_double(double value)
	throws BufferOverrunException
    {
	int size = DataConversion.sizeof_double(1);
	check_for_overrun(size);
	DataConversion.put_double(value, buffer, current_location);
	current_location += size;
    }
    public void put_short(short value)
	throws BufferOverrunException
    {
	int size = DataConversion.sizeof_short(1);
	check_for_overrun(size);
	DataConversion.put_short(value, buffer, current_location);
	current_location += size;
    }
    public void put_int(int value)
	throws BufferOverrunException
    {
	int size = DataConversion.sizeof_int(1);
	check_for_overrun(size);
	DataConversion.put_int(value, buffer, current_location);
	current_location += size;
    }
    public void put_long(long value)
	throws BufferOverrunException
    {
	int size = DataConversion.sizeof_long(1);
	check_for_overrun(size);
	DataConversion.put_long(value, buffer, current_location);
	current_location += size;
    }
    public void put_char(char value)
	throws BufferOverrunException
    {
	int size = DataConversion.sizeof_char(1);
	check_for_overrun(size);
	DataConversion.put_char(value, buffer, current_location);
	current_location += size;
    }
    public void put_byte(byte value)
	throws BufferOverrunException
    {
	int size = DataConversion.sizeof_byte(1);
	check_for_overrun(size);
	DataConversion.put_byte(value, buffer, current_location);
	current_location += size;
    }


    /*
     * put_TYPE(value[], start, count)
     */
    public void put_float(float value[], int start, int count)
	throws BufferOverrunException
    {
    	int size = DataConversion.sizeof_float(value.length);
	check_for_overrun(size);
	DataConversion.put_float(value, start, count, buffer, current_location);
	current_location += size;
    }
    public void put_double(double value[], int start, int count)
	throws BufferOverrunException
    {
    	int size = DataConversion.sizeof_double(value.length);
	check_for_overrun(size);
	DataConversion.put_double(value, start, count, buffer, current_location);
	current_location += size;
    }
    public void put_short(short value[], int start, int count)
	throws BufferOverrunException
    {
    	int size = DataConversion.sizeof_short(value.length);
	check_for_overrun(size);
	DataConversion.put_short(value, start, count, buffer, current_location);
	current_location += size;
    }
    public void put_int(int value[], int start, int count)
	throws BufferOverrunException
    {
    	int size = DataConversion.sizeof_int(value.length);
	check_for_overrun(size);
	DataConversion.put_int(value, start, count, buffer, current_location);
	current_location += size;
    }
    public void put_long(long value[], int start, int count)
	throws BufferOverrunException
    {
	int size = DataConversion.sizeof_long(value.length);
	check_for_overrun(size);
	DataConversion.put_long(value, start, count, buffer, current_location);
	current_location += size;
    }
    public void put_char(char value[], int start, int count)
	throws BufferOverrunException
    {
	int size = DataConversion.sizeof_char(value.length);
	check_for_overrun(size);
	DataConversion.put_char(value, start, count, buffer, current_location);
	current_location += size;
    }
    public void put_byte(byte value[], int start, int count)
	throws BufferOverrunException
    {
	int size = DataConversion.sizeof_byte(value.length);
	check_for_overrun(size);
	DataConversion.put_byte(value, start, count, buffer, current_location);
	current_location += size;
    }


    /*
     * sizeof_startpoint()
     */
    public static int sizeof_startpoint(Startpoint sp)
    {
	return (sp.sizeof());
    }
    public static int sizeof_startpoint(Startpoint[] sp,
					int start,
					int count)
    {
	int size = 0;
	for (int i = start; i < start + count; i++)
	{
	    size += sp[i].sizeof();
	}
	return (size);
    }


    /*
     * put_startpoint()
     */
    public void put_startpoint(Startpoint sp)
	throws BufferOverrunException
    {
	sp.put(this);
    }
    public void put_startpoint(Startpoint[] sp,
				   int start,
				   int count)
	throws BufferOverrunException
    {
	for (int i = start; i < start + count; i++)
	{
	    sp[i].put(this);
	}
    }

} /* class PutBuffer */
